﻿using ELTE.TicTacToeGame.Model;
using ELTE.TicTacToeGame.Persistence;
using ELTE.TicTacToeGame.ViewModel;

namespace ELTE.TicTacToeGame;

public partial class App : Application
{
    #region Private fields

    private ITicTacToeDataAccess _persistence;
    private ITicTacToeModel _model;
    private TicTacToeViewModel _viewModel;

    #endregion

    #region Constructors

    public App()
	{
		InitializeComponent();

        _persistence = new TicTacToeTextFileDataAccess(); // platformfüggetlen perzisztencia

        _model = new BasicTicTacToeModel(_persistence);
        _model.GameWon += new EventHandler<GameWonEventArgs>(Model_GameWon);
        _model.GameOver += new EventHandler(Model_GameOver);

        _viewModel = new TicTacToeViewModel(_model);
        _viewModel.LoadGame += new EventHandler(ViewModel_LoadGame); // kezeljük a nézetmodell eseményeit
        _viewModel.SaveGame += new EventHandler(ViewModel_SaveGame);

        BindingContext = _viewModel;
        MainPage = new AppShell();
	}

    #endregion

    #region Application life-cycle methods

    protected override Window CreateWindow(IActivationState? activationState)
    {
        Window window = base.CreateWindow(activationState);

        window.Created += async (s, e) =>
        {
            _model.NewGame(); // indításkor új játékot kezdünk

            // betöltjük a felfüggesztett játékot, amennyiben van
            try
            {
                await _model.LoadGameAsync(
                    Path.Combine(FileSystem.AppDataDirectory, "SuspendedGame"));
            }
            catch { }
        };

        window.Resumed += async (s, e) =>
        {
            // betöltjük a felfüggesztett játékot, amennyiben van
            try
            {
                await _model.LoadGameAsync(
                    Path.Combine(FileSystem.AppDataDirectory, "SuspendedGame"));
            }
            catch { }
        };

        window.Stopped += async (s, e) =>
        {
            // elmentjük a jelenleg folyó játékot
            try
            {
                await _model.SaveGameAsync(
                    Path.Combine(FileSystem.AppDataDirectory, "SuspendedGame"));
            }
            catch { }
        };

        return window;
    }

    #endregion

    #region Model event handlers

    /// <summary>
    /// Játék megnyerésének eseménykezelése.
    /// </summary>
    private async void Model_GameWon(object? sender, GameWonEventArgs e)
    {
        switch (e.Player)
        {
            case Player.PlayerO:
                await MainPage!.DisplayAlert("Játék vége!", "A kör játékos győzött!", "OK");
                break;
            case Player.PlayerX:
                await MainPage!.DisplayAlert("Játék vége!", "A kereszt játékos győzött!", "OK");
                break;
        }
        _model.NewGame(); // új játékot kezdünk
    }

    /// <summary>
    /// Játék végének eseménykezelése.
    /// </summary>
    private async void Model_GameOver(object? sender, EventArgs e)
    {
        await MainPage!.DisplayAlert("Döntetlen játék!", "Játék vége!", "OK");
        _model.NewGame(); // új játékot kezdünk
    }

    #endregion

    #region ViewModel event handlers

    /// <summary>
    /// Játék betöltésének eseménykezelője.
    /// </summary>
    private async void ViewModel_LoadGame(object? sender, System.EventArgs e)
    {
        try
        {
            await _model.LoadGameAsync(
                Path.Combine(FileSystem.AppDataDirectory, "LastGame"));
        }
        catch { }
    }

    /// <summary>
    /// Játék mentésének eseménykezelője.
    /// </summary>
    private async void ViewModel_SaveGame(object? sender, System.EventArgs e)
    {
        try
        {
            await _model.SaveGameAsync(
                Path.Combine(FileSystem.AppDataDirectory, "LastGame"));
        }
        catch
        {
            await MainPage!.DisplayAlert("Hiba!", "A mentés sikertelen volt.", "OK");
        }
    }

    #endregion
}
